/*	DOS Stamp ADC functions
	(C)1999 Bagotronix, Inc.
	Author: Ivan Baggett

	Written in Borland C v3.0.
	For Maxim MAX197 ADC chip
*/

#include <dos.h>

#include "stdinc.h"
#include "dosstamp.h"
#include "dsadc.h"


/****************************************************************************
ADCput():  starts an ADC conversion
		args:
			1) channel# (0 -7)
			2) configuration of channel
		uses:  ADC, PIO18
		comments:  incorrect results will occur if arguments are out
			of range
****************************************************************************/
void ADCput (UBYTE channel, UBYTE config) {
	outportb (ADC_BASE, channel | config | 0x40);
}


/****************************************************************************
ADCget():  reads an ADC conversion result
		args:  none
		returns:  the conversion result, zero extended if unipolar,
			sign extended if bipolar
		uses:  ADC, PIO18
		comments:  reading the result clears the INT# output of the ADC
****************************************************************************/
UWORD ADCget (void) {
	return (inport (ADC_BASE));
}


/****************************************************************************
ADCready():  checks if an ADC conversion result is ready
		args:  none
		returns:  0 if not ready, non-zero if ready
		uses:  PIO18
		comments:  PIO18 must be configured as a general purpose input,
			with or without pullup
****************************************************************************/
BOOLEAN ADCready (void) {
	return (!(inport (PIO1_DATA) & 0x0004));
}
