/*	DOS Stamp Test program
	(C)1999 Bagotronix, Inc.
	Author: Ivan Baggett

	Written in Borland C v3.0.
	Use this program as an example on how to operate the DOS Stamp.

	v1.1	8/17/99 added PIO0_MASK and PIO1_MASK to fix input reporting bug.
*/

#include <dos.h>
#include <stdio.h>

#include "stdinc.h"
#include "dosstamp.h"
#include "dsrtc.h"
#include "dsadc.h"
#include "dscomm.h"

#define PIO0_MASK 0x3c03
#define PIO1_MASK 0xd83f


BOOLEAN done = FALSE;
long ticks;

WORD waitforkey (void) {
	WORD c;

	while (!kbhit());
	if (!(c = getch()))
		c = getch();
	return c;
}

BOOLEAN TestPIO (void) {
	BOOLEAN done;
	WORD c, i;
	UWORD pio0, pio1, tpio0, tpio1;

	printf ("\nTesting PIOs as outputs driving low. Press a key to advance...");
	done = FALSE;
	i = 0;
	while (!done) {
		waitforkey();
		PIOconfig (i, PIOCFG_OUTPUT, 0);
		printf ("\n%d, PIO0 = %04x, PIO1 = %04x", i, inport (PIO0_DATA),
			inport (PIO1_DATA));
		++i;
		if (i > 31)
			done = TRUE;
	}

	printf ("\nTesting PIOs as outputs driving high.  Press a key to advance...");
	done = FALSE;
	i = 0;
	while (!done) {
		waitforkey();
		PIOconfig (i, PIOCFG_OUTPUT, 1);
		printf ("\n%d, PIO0 = %04x, PIO1 = %04x", i, inport (PIO0_DATA),
			inport (PIO1_DATA));
		++i;
		if (i > 31)
			done = TRUE;
	}

	printf ("\nTesting PIOs as inputs.  Operate tester switches, press a key to quit...");
	done = FALSE;
	for (i = 0; i < 32; i++) {
		PIOconfig (i, PIOCFG_INPUT, 1);
	}
	// get initial values
	pio0 = inport (PIO0_DATA & PIO0_MASK);
	pio1 = inport (PIO1_DATA & PIO1_MASK);
	while (!done) {
		if ((tpio0 = inport (PIO0_DATA) & PIO0_MASK) != pio0) {
			pio0 = tpio0;
			printf ("\nPIO0 = %04x", pio0);
		}
		if ((tpio1 = inport (PIO1_DATA) & PIO1_MASK) != pio1) {
			pio1 = tpio1;
			printf ("\nPIO1 = %04x", pio1);
		}
		if (kbhit()) {
			waitforkey();	// flush keystroke
			done = TRUE;	// and quit this test
		}
	}
	return TRUE;
}


void interrupt RTC_ISR (void) {
	inportb (RTC_REGC);		// clear PF flag bit
	++ticks;
	SEOI(INT3_VEC);
}


BOOLEAN TestRTC (void) {
	WORD i, j, c;
	UWORD k;
	BOOLEAN done = FALSE;
	BYTE s[128];
	UWORD hr, min, sec;
	UWORD mon, date, yr, day;
	LONG lticks;

	RTCinit();
	RTCon();
	RTCalarmcontrol (FALSE, FALSE, FALSE);

	printf ("\nRTC tests:");
	while (!done) {
		printf ("\n1 - RTC dump");
		printf ("\n2 - RTC set time and date");
		printf ("\n3 - RTC read time and date");
		printf ("\n4 - RTC RAM test");
		printf ("\n5 - RTC power-down/up and kickstart test");
		printf ("\n6 - RTC periodic interrupt test");
		printf ("\nother - main menu");

		c = waitforkey();

		switch (c) {
			case '1':	// RTC dump
				printf ("\nDump of RTC registers and memory:");
				for (j = 0; j < 16; j++) {
					printf ("\n");
					for (i = 0; i < 16; i++) {
						k = inportb (RTC_BASE + j * 16 + i);
						printf ("%02x ", k);
					}
				}
				break;
			case '2':	// RTC set time and date
				printf ("\nEnter the time as HH MM SS -> ");
				gets (s);
				sscanf (s, "%d %d %d", &hr, &min, &sec);
				RTCsettime (hr, min, sec);

				printf ("\nEnter the date as MM DD YYYY -> ");
				gets (s);
				sscanf (s, "%d %d %d", &mon, &date, &yr);
				printf ("\nEnter the day as a digit from 1 to 7 (Sunday = 1) -> ");
				gets (s);
				sscanf (s, "%d", &day);
				RTCsetdate (mon, date, yr, day);
				break;
			case '3':	// RTC read time
				RTCgettime (&hr, &min, &sec);
				printf ("\nRTC time:  %02d:%02d:%02d", hr, min, sec);
				RTCgetdate (&mon, &date, &yr, &day);
				printf ("\nRTC date:  %02d/%02d/%04d, day=%d", mon, date,
					yr, day);
				break;
			case '4':
				printf ("\nTesting RTC RAM...");
				if (RTCRAMtest())
					printf ("OK");
				else
					printf ("BAD");
				break;
			case '5':	// power-down/up and kickstart test
				// Set alarm time for 10 seconds from now
				RTCgettime (&hr, &min, &sec);
				sec += 10;
				if (sec > 59) {
					sec -= 60;
					++min;
				}
				if (min > 59) {
					min = 0;
					++hr;
				}
				if (hr > 23)
					hr = 0;
				RTCsetalarmtime (hr, min, sec);
				// activate wake-up and kickstart
				RTCalarmcontrol (FALSE, FALSE, TRUE);
				RTCpowerdown();
				break;
			case '6':	// periodic interrupt test
				// prepare INT3 for activity
				lticks = ticks = 0L;
				IRQinit (INT3_VEC, RTC_ISR, 7, EDGE);
				RTCsetperiod (RTC_16HZ);
				RTCalarmcontrol (TRUE, FALSE, FALSE);
				unmask (INT3_VEC);
				printf ("\nPress a key to quit...\n");
				while (!done) {
					if (kbhit()) {
						done = TRUE;
						waitforkey();
					}
					disable();
					lticks = ticks;
					enable();
					printf ("%ld   ", lticks);
				}
				mask (INT3_VEC);
				RTCalarmcontrol (FALSE, FALSE, FALSE);
				done = FALSE;
			default:
				done = TRUE;
				break;
		}
	}
	return TRUE;
}

BOOLEAN TestADC (void) {
	BOOLEAN done = FALSE;
	UWORD u;
	WORD i, c;
	double v;

	printf ("\nPress a key to scan ADC channels, 'X' to quit...");
	while (!done) {
		c = waitforkey();
		if (c == 'x' || c == 'X')
			done = TRUE;
		else {
			for (i = 0; i < 8; i++) {
				ADCput (i, ADC_0_TO_5);
				while (!ADCready());	// wait for conversion
				u = ADCget();
				v = (double)u * 1.2207 / 1000.0;
				printf ("\nADC CH%d = %04x, %1.3lf V", i, u, v);
			}
		}
	}
	return TRUE;
}

BOOLEAN TestComm (void) {
	BOOLEAN done = FALSE;
	WORD c = 0;
	UBYTE b;
	UWORD u;

	// configure PIO pins as serial port functions (normal)
	PIOconfig (27, PIOCFG_NORMAL, 1);
	PIOconfig (28, PIOCFG_NORMAL, 1);

	// configure PIO19 (GPIO12) as output with initial value of 0
	// to activate the RS-485 driver
	PIOconfig (19, PIOCFG_OUTPUT, 0);

	SERsetbaud (1, B9600);		// set baud rate
	SERinit (1, FALSE, FALSE, FALSE);	// init serial port 1 for polling

	printf ("\nTesting serial communications on COM2.");
	printf ("\nIf RS-232, install loopback connection between RX and TX.");
	printf ("\nIf RS-485, no loopback connection is required, echo is normal.");
	printf ("\nPress keys to send characters, or 'X' to quit...\n");
	while (!done) {
		if (kbhit()) {
			c = waitforkey();
			while (!SERxmtrready (1));	// wait for xmtr if not ready
			SERputc (1, c);
		}
		if (SERrcvrready (1)) {
			b = SERgetc (1);
			printf ("%c", b);
		}
		if (c == 'x' || c == 'X')
			done = TRUE;
	}

/*	Leave PIO19 (GPIO12) configured as output, but write a 1 to it to disable
	the RS-485 driver.  You could also just as easily reconfigure it as an input
	with pull-up using the PIOconfig() function.
*/
	disable();
	u = inport (PIO1_DATA);
	u |= 0x0008;			// bit 3 a.k.a. PIO19 a.k.a. GPIO12 = 1
	outport (PIO1_DATA, u);
	enable();

	return TRUE;
}


void PrintMenu (void) {

	printf ("\n\nChoose a test:");
	printf ("\n1 - PIO (general purpose I/O)");
	printf ("\n2 - RTC (real time clock)");
	printf ("\n3 - ADC");
	printf ("\n4 - Serial communications");
	printf ("\nX - Exit (quit)");
}

int main (void) {
	WORD c;

	printf ("\nThis is DOS Stamp(tm) from Bagotronix, Inc.");

	while (!done) {
		PrintMenu();
		c = waitforkey();
		switch (c) {
			case 'x':
			case 'X':
				done = TRUE;
				break;
			case '1':	// test PIOs as general purpose I/O
				TestPIO();
				break;
			case '2':	// test RTC
				TestRTC();
				break;
			case '3':	// test ADC
				TestADC();
				break;
			case '4':	// test serial comm
				TestComm();
				break;
			default:
				break;

		}
	}

	return 0;
}